      SUBROUTINE SPLIN2(FMAX,LOG,START,IERR)
C
CC  calculates two-dimensional spline out of background image
C
C************************ FFORM VERSION 1.1 ************  7-NOV-89 15:13
C
CA  author : GRH        date: 1-DEC-1986 18:32
CU  update : CRI        date: 2-FEB-1988 12:08
CU  update : CRI        date: 2-FEB-1988 12:27
CU  update : PTK        date: 16-SEP-1988 16:37
CU  update : PTK        date: 11-OCT-1988 18:15
CU  update : PTK        date: 3-NOV-1989 10:01
CU  update : PTK        date: 7-NOV-1989 15:15  
CU  update : PTK        date: 19-FEB-1990 08:04
CU  update : PTK        date: 17-JAN-1992 16:56 logarithmic spline interp.
CU  update : PTK        date: 03-JUN-1992 18:26 correct bug 
CU  update : PTK        date: 03-DEC-1992 14:36 SASS update
CU  update : PTK        date: 15-DEC-1992 10:29
CU  update : EXSAS              date: 03-AUG-1993 16:52
C
CT  status: not tested
C
C   general description 
CG  routine takes the real representation of the binned background image
CG  and produces the bicubic spline representation.
C
C   call_var.          type I/O description 
CP  FMAX                R4    o maximum of the function
CP  LOG                 L4  i   logarithmic spline interpolation flag
CP  START               L4  i   set start values flag
CP  IERR                I4    o error code
C
C   include_block name          description 
CI  RCOMMON:CDBIMG.CMN          DB image arrays parameter common block
CI  RCOMMON:CDBNAG.CMN          util. var. for spline fit common block
CI  RCOMMON:CDBNAG.INC          util. var. for spline fit common block, DATA
CI  RCOMMON:CGENL.CMN           general parameter common block
C
C   routines_called    type     description 
CR  ERPLPS              SR       calculates spline parameters
CR  CALBCS              SR       calculates spline values
CR  LPSROS              SR       sorts data into proper rectangles
CR  WRERR               SR       write error message
C
C   extensions/system calls     description 
CX 
C
C***********************************************************************
C
C   variables   meaning
C
      use global

      IMPLICIT NONE
C
      REAL        DELTAX,DELTAY
      INTEGER     I, IERR, ISTAT, NP1, NQX, NQY, NRANK, L
      LOGICAL     WRITE,LOG,START
      CHARACTER*8   RTNAME
      DOUBLE PRECISION
     &        XMIN,XMAX,YMIN,YMAX,FMIN,FMAX
C
      INCLUDE 'include/cgenl.cmn'
!     INCLUDE 'include/cdbimg.cmn'
      INCLUDE 'include/cdbnag.cmn'
C
      SAVE
C     INCLUDE 'RCOMMON:CDBNAG.INC'
      DATA          RTNAME /'SPLIN2'/
      DATA          WRITE  /.FALSE./
C
CS!   initial steps
C-1
      IERR = 0
C
      IF (START) THEN     
        START = .FALSE.
C
C   prepare input for LPSROS
C
        XMIN =   1.D21
        XMAX =  -1.D+21
        YMIN =   1.D21
        YMAX =  -1.D+21
        FMIN =   1.D21
        FMAX =  -1.D+21
C 
C find extrema

        DO I = 1,NPTS
          IF(XNAG(I).LT.XMIN)  XMIN = XNAG(I)
          IF(XNAG(I).GT.XMAX)  XMAX = XNAG(I)
          IF(YNAG(I).LT.YMIN)  YMIN = YNAG(I)
          IF(YNAG(I).GT.YMAX)  YMAX = YNAG(I)
C         IF(FNAG(I).LT.FMIN.AND.FNAG(I).NE.0.D0)  FMIN = FNAG(I)
          IF(FNAG(I).LT.FMIN.AND.FNAG(I).GT.0.D0)  FMIN = FNAG(I)
          IF(FNAG(I).GT.FMAX)  FMAX = FNAG(I)
        ENDDO
C
CDOE        IF (FMAX.GT.10.*FMIN) THEN
C          LOG=.TRUE.
C          DO I=1,NPTS
C            IF(FNAG(I).GT.0.)THEN
C              WNAG(I)=WNAG(I)*FNAG(I)*2.303
C              FNAG(I)  = DLOG10(FNAG(I))
C            ELSE
C              WNAG(I)=WNAG(I)*FNAG(I)
C              FNAG(I)=DLOG10(FMIN)
C            ENDIF
C          ENDDO
C          FMIN=DLOG10(FMIN)
C          FMAX=DLOG10(FMAX)
C          CALL STTPUT(' Logarithmic spline interpolation ',ISTAT)
C        ELSE
CHB          LOG=.FALSE.
CDOE        ENDIF
C
CW!   set equidistant spline knots
C
        NQX = NPX-8
        NQY = NPY-8
        DELTAX = (XMAX-XMIN)/REAL(NQX+1)
        DELTAY = (YMAX-YMIN)/REAL(NQY+1)
        DO I = 1,NQX
          XL(I+4) = XMIN+I*DELTAX
        ENDDO
        DO I = 1,NQY
          YM(I+4) = YMIN+I*DELTAY
        ENDDO

C     add the two extreme points (0,0), (IPA_X,IPA_Y)
C     to enable to calculate values outside the original rectangle

        NP1       = NPTS+1
        XNAG(NP1) = 0.D0
        YNAG(NP1) = 0.D0
        IF (LOG) THEN
          FNAG(NP1) = FMIN
        ELSE
          FNAG(NP1) = 0.D0
        ENDIF
        WNAG(NP1) = 0.D0
        NP1       = NPTS+2
        XNAG(NP1) = DBLE(IPA_X)
        YNAG(NP1) = DBLE(IPA_Y)
        IF (LOG) THEN
          FNAG(NP1) = FMIN
        ELSE
          FNAG(NP1) = 0.D0
        ENDIF
        WNAG(NP1) = 0.D0
C
C     call LPSROS to sort data into proper rectangles
C
        ISTAT = -1
        CALL LPSROS(NPY,NPX,YM,XL,NP1,YNAG,XNAG,IPOINT,NPOINT,
     #              IADRES,NADRES,ISTAT)
        IF (ISTAT.GT.1) THEN
          IERR = 3
          call error("SplineFitError","Error in LPSROS")
        ENDIF
      ENDIF
C
      IF (IERR.LE.1) THEN
C
C    call ERPLPS to calculate spline parameters
C
        ISTAT = -1
        CALL ERPLPS(NP1,NPY,NPX,YNAG,XNAG,FNAG,WNAG,YM,XL,IPOINT,
     #              NPOINT,DL,C,NC,WORK,NWORK,EPS,SIGMA,NRANK,ISTAT)
        IF (ISTAT.GT.1) THEN
           IERR = 3
           call error ("SplineFitError","Error in ERPLPS")
        ENDIF
      ENDIF

      IF (IERR.LE.1) THEN
C
C  call CALBCS to calculate spline representation
C
        ISTAT = -1
        CALL CALBCS(NP1,YNAG,XNAG,FFNAG,NPY,NPX,YM,XL,
     #              NPOINT,IPOINT,NC,C,ISTAT)
        IF (ISTAT.GT.1) THEN
           IERR = 3
           call error("SplineFitError","Error in CALBCS")
        ENDIF
      ENDIF
C
CH! DB  -  BACKGROUND
C
      RETURN
      END
